<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Validation\Rules;
use Illuminate\Validation\Rule;
use Illuminate\Validation\Rules\Password;
use App\Models\User;

class UserController extends Controller
{
    /**
     * All Users list
     */
    public function index() {
        $users = User::all();
        return view('admin.users.list',['users' => $users]);
    }

    /**
     * User create form
     */
    public function create() {
        return view('admin.users.create');
    }

    /**
     * Handle an incoming user create request.
     *
     */
    public function store(Request $request): RedirectResponse
    {

        $request->validate([
            'name' => ['required', 'string', 'max:255'],
            'email' => ['required', 'string', 'lowercase', 'email', 'max:255', 'unique:'.User::class],
            'password' => ['required', 'confirmed', Rules\Password::defaults()],
            'role' => ['required', 'string', 'max:255'],
            'status' => ['required', 'string', 'max:255'],
        ]);

        $user = User::create([
            'name' => $request->name,
            'email' => $request->email,
            'password' => Hash::make($request->password),
            'role' => $request->role,
            'status' => $request->status,
            'created_by' => $request->user()->id,
        ]);

        return redirect()->back()->with('success', 'User created Successfully.');
    }

    /**
     * Display the Edit View
     */
    public function edit($id) {

        $user = User::findOrFail($id);

        if(!$user) {
            return redirect()->route('admin.users')->with('error', 'User not found.');
        }

        return view('admin.users.edit',['user' => $user]);
    }

    /**
     * Update User
     */
    public function update(Request $request, $id) {

        $user = User::findOrFail($id);

        if(!$user) {
            return redirect()->route('admin.users')->with('error', 'User not found.');
        }

        $validated = $request->validate([
            'name' => 'required|string|max:255',
            'email' => [
                'required',
                'email',
                'max:255',
                Rule::unique('users')->ignore($user->id),
            ],
            'status' => ['required', 'string', 'max:255'],
            'role' => ['required', 'string', 'max:255'],
        ]);

        $user->update($validated);
        $user->update([
            'updated_by' => $request->user()->id,
        ]);


        if($request->password) {
            $validatePassword = $request->validate([
                'password' => ['required', Password::defaults()],
            ]);

            $user->update([
                'password' => Hash::make($validatePassword['password']),
            ]);
        }

        return redirect()->back()->with('success', 'User updated Successfully.');
    }

    /**
     * Delete the user.
     */
    public function delete($id) {
        $user = User::findOrFail($id);

        if(!$user) {
            return redirect()->route('admin.users')->with('error', 'User not found.');
        }

        $user->delete();
        return redirect()->back()->with('success', 'User deleted Successfully.');
    }
}
